/* eslint-disable prefer-destructuring */
/* eslint-disable no-async-promise-executor */
/* eslint-disable prefer-rest-params */

import { loadLocusLibrary } from './lclib';

loadLocusLibrary().then((res:any)=>{
  // LocusWasm.GetLibraryVersions().then(res=>{
  //   console.log(JSON.stringify(res));
  // });
});

export class LocusLib {
  static callWasm(wasmCmd: string, wasmParams: any) {
    return new Promise(async (resolve, reject) => {
      // console.debug('>>>>>>>>>>>>>> wasmParams: ', wasmParams);
      if (Array.isArray(wasmParams)) {
        wasmParams = wasmParams[0];
      }

      // console.debug('>>>>>>>>>>>>>> wasmParams: ', wasmCmd, wasmParams);

      if(typeof globalThis['CallLclib'] !== "function") return;
      
      const resultString = await globalThis['CallLclib'](wasmCmd, JSON.stringify(wasmParams));
      // console.debug('>>>>>>>>>>>>>> resultString: ', resultString);
      const result = JSON.parse(resultString);

      // console.debug(">>> result: ", result);

      if (result.Error && result.Error.Code != 0) {
        reject(result.Error);
      } else {
        resolve(result.Result);
      }

      // console.debug(`WASM ${wasmCmd}() call: `, [...wasmParams], window[wasmCmd]);

      // window[wasmCmd](...[...wasmParams], function (data: any, err: any) {
      //   const error = JSON.parse(err);
      //   if (err && error.Error == 0) {
      //     console.debug(`WASM ${wasmCmd}() callback: `, data);
      //     resolve(data);
      //   } else {
      //     console.error(`WASM ${wasmCmd}() callback: `, err);
      //     resolve(null);
      //   }
      // });
    });
  }

  //=================================================================================
  static GetLibraryVersions() {
    return LocusLib.callWasm('GetLibraryVersions', {});
  }
  static ConvertToData(value, valueType) {
    return LocusLib.callWasm('ConvertToData', { value, valueType });
  }
  static ConvertDataTo(dataString, retType) {
    return LocusLib.callWasm('ConvertDataTo', { dataString, retType });
  }
  static ConvertStringToData(str) {
    return LocusLib.callWasm('ConvertStringToData', str);
  }
  static ConvertDataToString(dataString) {
    return LocusLib.callWasm('ConvertDataToString', dataString);
  }
  static ConvertAddressToHex(addr) {
    return LocusLib.callWasm('ConvertAddressToHex', addr);
  }
  static ConvertHexToAddress(hexString) {
    return LocusLib.callWasm('ConvertHexToAddress', hexString);
  }
  static ConvertAddressToData(addr) {
    return LocusLib.callWasm('ConvertAddressToData', addr);
  }
  static ConvertDataToAddress(dataString) {
    return LocusLib.callWasm('ConvertDataToAddress', dataString);
  }
  static ConvertHexToData(encoded) {
    return LocusLib.callWasm('ConvertHexToData', encoded);
  }
  static ConvertDataToHex(dataString) {
    return LocusLib.callWasm('ConvertDataToHex', dataString);
  }
  static ConvertBase32ToHex(encoded) {
    return LocusLib.callWasm('ConvertBase32ToHex', encoded);
  }
  static ConvertHexToBase32(hexString) {
    return LocusLib.callWasm('ConvertHexToBase32', hexString);
  }
  static ConvertBase32ToData(encoded) {
    return LocusLib.callWasm('ConvertBase32ToData', encoded);
  }
  static ConvertDataToBase32(dataString) {
    return LocusLib.callWasm('ConvertDataToBase32', dataString);
  }
  static ConvertBase64ToHex(encoded) {
    return LocusLib.callWasm('ConvertBase64ToHex', encoded);
  }
  static ConvertHexToBase64(hexString) {
    return LocusLib.callWasm('ConvertHexToBase64', hexString);
  }

  // static getWasmVersion() {
  //   return LocusWasm.callWasm('GetWasmVersion', [...arguments]);
  // }

  // static createAccountAndEncrypt(password) {
  //   return LocusWasm.callWasm('CreateAccountAndEncrypt', [...arguments]);
  // }

  static CreateAccountAndKeystore(
    passwordMaster: string,
    passwordNormal: string,
    mkeyAlgoNormal: string,
    nkeyAlgoNormal: string
  ) {
    return LocusLib.callWasm('CreateAccountAndKeystore', {
      passwordMaster,
      passwordNormal,
      mkeyAlgoNormal,
      nkeyAlgoNormal
    });
  }

  static LoadMasterKeystore(PassStr: string, KsJson: string) {
    return LocusLib.callWasm('LoadMasterKeystore', { PassStr, KsJson });
  }

  static LoadNormalKeystore(PassStr: string, KsJson: string) {
    return LocusLib.callWasm('LoadNormalKeystore', { PassStr, KsJson });
  }

  static Sign(Sk: any, Message: any) {
    return LocusLib.callWasm('Sign', { Sk, Message });
  }

  static VerifyTx(Pk: any, Message: any, Sig: any) {
    return LocusLib.callWasm('VerifyTx', { Pk, Message, Sig });
  }

  static CreateNormalKey(addrStr: any, keyAlgo: string) {
    return LocusLib.callWasm('CreateNormalKey', { addrStr, keyAlgo });
  }

  static CreateMasterKeystore(AddrStr: any, Password: any, MskStr: any, MpkStr: any) {
    return LocusLib.callWasm('CreateMasterKeystore', { AddrStr, Password, MskStr, MpkStr });
  }

  static CreateNormalKeystore(
    AddrStr: any,
    Password: any,
    NskStr: any,
    NpkStr: any,
    MpkStr: any = '',
    KeySign: any = ''
  ) {
    return LocusLib.callWasm('CreateNormalKeystore', { AddrStr, Password, NskStr, NpkStr, MpkStr, KeySign });
  }

  static SignByMasterKey(msk: string, message: string) {
    return LocusLib.callWasm('SignByMasterKey', { msk, message });
  }

  static CreateTxActivateToken(jsonTx: string) {
    return LocusLib.callWasm('CreateTxActivateToken', jsonTx);
  }

  static DecodeTxs(txs) {
    return LocusLib.callWasm('DecodeTxs', { txs });
  }

  static CompileCoreScript(code) {
    return LocusLib.callWasm('CompileCoreScript', { code });
  }

  static DisCompileCoreScript(code) {
    return LocusLib.callWasm('DisCompileCoreScript', { code });
  }

  static TestCoreScript(scriptProvide, scriptAccept, txDataProvide, txDataAccept) {
    return LocusLib.callWasm('TestCoreScript', {
      scriptProvide,
      scriptAccept,
      txDataProvide,
      txDataAccept
    });
  }

  // Type는 /lctest/types/cstypes.go CsDataType 값 참고
  static EncodeTxNumber(Number, Type) {
    return LocusLib.callWasm('EncodeTxNumber', { Number, Type });
  }

  static EncodeTxCurrency(currency) {
    return LocusLib.callWasm('EncodeTxCurrency', { currency });
  }

  /*
  Type:
    0: sha256
    1: blake2b256
  */
  static Hash(data, type) {
    return LocusLib.callWasm('Hash', { data, type });
  }

  static GetDefFromCoreScript(code) {
    return LocusLib.callWasm('GetDefFromCoreScript', { code });
  }
}
