import { LocusLib } from 'locuschain-lib';
import { ethers } from '../ethers.js-5.5.4-24bytes-addr/ethers.esm.min';

export const decodeResult = async (func: any, result: any): Promise<{ value: any; name: string; type: string }[]> => {
  const hexString = (await LocusLib.ConvertDataToHex(result)) as string;
  let iface = new ethers.utils.Interface([func]);
  const decodeFunctionResult = iface.decodeFunctionResult(
    func.name,
    hexString.startsWith('0x') ? hexString : `0x${hexString}`
  );
  let resultObj = JSON.parse(JSON.stringify(decodeFunctionResult));

  let i = 0;
  await func.outputs.reduce(async (previousPromise: Promise<null>, out: any) => {
    await previousPromise;
    return new Promise(async (resolve, reject) => {
      let value = resultObj[i];
      if (out.type === 'address') {
        value = await LocusLib.ConvertHexToAddress(resultObj[i].toLowerCase());
      } else if (resultObj[i].type) {
        if (resultObj[i].type == 'BigNumber') {
          value = ethers.BigNumber.from(resultObj[i]).toString();
        }
      }
      resultObj[i] = { value, ...out };
      i++;
      resolve(null);
    });
  }, Promise.resolve());

  return resultObj;
};

export const removeHexPrefix = hexString => {
  if (!hexString) {
    return hexString;
  }
  if (hexString.toLowerCase().startsWith('0x')) {
    return hexString.substring(2);
  }
  return hexString;
};

export const convertJsonToFuncString = item => {
  let inputString = item.inputs.map(x => `${x.type}`).join(',');

  if (item.name) {
    return `${item.name}(${inputString})`;
  } else {
    return `constructor(${inputString})`;
  }
};

export const encodeArgHex = async (inputs: any[], paramOriginValues?: any[]) => {
  let paramTypes: any[] = [];
  let paramValues: string[] = [];

  let idx = 0;
  await inputs.reduce(async (previousPromise, x) => {
    await previousPromise;
    return new Promise(async (resolve, reject) => {
      let value = paramOriginValues ? paramOriginValues[idx++] : x.value;
      if (x.type === 'address') {
        value = (await LocusLib.ConvertAddressToHex(value)) as string;
        if (value != null) {
          value = '0x' + value;
          // console.log(x.value + ' -> ' + value);
        }
      }

      paramTypes.push(x.type);
      paramValues.push(value);
      // console.log('(' + x.type + ':' + x.value + ')');
      resolve(null);
    });
  }, Promise.resolve());

  if (paramTypes.length === 0) {
    return '';
  } else {
    return ethers.utils.defaultAbiCoder.encode(paramTypes, paramValues);
    // return web3.eth.abi.encodeParameters(paramTypes, paramValues);
  }
};
